# -- BEGIN COPYRIGHT --
# IBM SecureWay Firewall Version 4.1
# Licensed Materials - Property of IBM
#
# 5697-F48
#
# (C) Copyright IBM Corp. 1985, 1994, 1999 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -- END COPYRIGHT --

# Program File: Logfile.pl

###########################################
#
# logfile adapter initialisation
#
###########################################

# configuration information.

%CfgTable = (
	"ServerName",		'',
	"PortNo",		'0',
	"CONFfile",		'',
	"PollInterval",		'1',
	"LOGFILES",		'',
	"ConnectAttempts",	'5',
	"MaxLogSize",		'1024',
	"KeepOldLogs",		'true',
	"Trace",		'true',
	"StartAt",		'0',
	"EOFfile",		'eofFile.dne',
);

#

%logfileTable = (
    "LOGFILE0" => {
	"name"    => NULL,
	"size"    => NULL,
	"inode"   => NULL,
    },
);

###########################################
#
# format table. 
#
# Remark: for the regexpr, DO NOT put the heading and trailing slash!!!
#         i.e.: /^ a regular expression $/
###########################################

%formatTable = (
    "rs"      => "\n",		# Record separator (newline)
    "multil"  => 0,		# Multi line match 0 means NO
    "regexpr" => '([A-Za-z]*) *([0-9]*) ([0-9:]*) ([A-Za-z0-9]*)*[A-Za-z: ]*([0-9]*);([0-9]*):[ ]*([A-Za-z0-9]*);(.......)(.);(.*)',
    "nvar"    => 10,		# number of variable the regexpr will produce (MAX 9)
    "varname" => [ "month", "day", "time", "hostname", "year", "rc", "id", "code", "codeMod", "message" ],
);

###########################################
###########################################

# the specific event contains the following default fields:
# "LOGFILE" that contains the adpater specific name 
# "complete" that contains the complete message.

%specificEvent = ();

###########################################
#
# logfile adapter functions
#
###########################################

# --------------------------------------------------------------------------
# Function: RotateFile()
#
# Description: check if the log file has grown to large. If so, roll
#   over to a new log file.
#
# Note: not currently used
# --------------------------------------------------------------------------

sub RotateFile
{
  local($logfile) = @_;
  local($fsize);
  my $i     = 0;
  my $old_f = $logfileTable{$logfile}{name};

  $fsize=(stat($old_f))[7];

  if($fsize > $CfgTable{MaxLogSize})
  {
    if($CfgTable{KeepOldLogs} eq "true")
    {
      $i++;

      while($i < 1000)
      {
        $new_f = $old_f . $i;
        if (-e $new_f) { $i++; }
        else
        {
          close($logfile);
          rename $old_f, $new_f || &myDie("Cannot rename $old_f to $new_f");
          open(NEWF, ">$old_f");
          close(NEWF);
	  open ($logfile, "<$logfileTable{$logfile}{name}") || &myDie("Cannot open logfile: $logfileTable{$logfile}{name}");
          return;
        } 
      }
    }
    else
    {
      open(NEWF, ">$old_f");
      close(NEWF);
    }
  }
}

# --------------------------------------------------------------------------
# Function: InitSpecific()
#
# Description: called by main() to open the logfile
#
# initializes the configuration 
# --------------------------------------------------------------------------

sub InitSpecific
{
  local($logfile,$n);
  local(@stat, @logfiles, $size);

  # split logfiles name and fill $logfileTable variable

  @logfiles=split(';', $CfgTable{LOGFILES});

  for ($n=0; $n < @logfiles; $n++)
    { $logfileTable{LOGFILE . $n}{name} = $logfiles[$n]; }

  foreach $logfile (keys %logfileTable)
  {
    # check that the logfile specified in the .conf file exist

    if (! -f $logfileTable{$logfile}{name})
    {
      &Trace($Level{MINOR}, "logfile: $logfileTable{$logfile}{name} does not exist\n");
      $logfileTable{$logfile}{size} = -1;
      next;
    }

#   &Trace($Level{VERBOSE}, "$logfile is $logfileTable{$logfile}{name}\n");

    open ($logfile, "<$logfileTable{$logfile}{name}") || &myDie("Cannot open logfile: $logfileTable{$logfile}{name}");

    @stat = stat($logfileTable{$logfile}{name});
	
#   &Trace($Level{VERBOSE},"$logfile is $stat[7] bytes.\n");

    # save the logfile inode

    $logfileTable{$logfile}{inode} = $stat[1];

    # ------------------------------------
    # defined in conf file: 'StartAt'
    #
    # -1 = start at beginning of log file
    #  0 = start where we stopped reading
    #  1 = start at current EOF

    if ($CfgTable{StartAt} eq "-1")
      { $logfileTable{$logfile}{size} = 0; }
    elsif ($CfgTable{StartAt} eq "1")
      { $logfileTable{$logfile}{size} = $stat[7]; }
    else
      {
        &getEofOffset($CfgTable{EOFfile}, *size);
        $logfileTable{$logfile}{size} = $size;
      }

    &Trace($Level{MINOR}, "start at: $logfileTable{$logfile}{size}\n");
  }
}

# --------------------------------------------------------------------------
# Function: ReadSpecific()
#
# Description: read a record from the logfile and tokenize it
#
# return code:  0  no new event 
# 		1  new event read 
#		   And %specificEvent{} table contains the event.
# --------------------------------------------------------------------------

sub ReadSpecific
{
  local($logfile);
  local(@stat);

  foreach $logfile (keys %logfileTable)
  {
    while (&readLogFile($logfile))
    {
      &createSpecificEvent($logfile);
      return(1);
    }

#   &RotateFile($logfile);
  }

  return(0);
}

# --------------------------------------------------------------------------
# Function: FinishSpecific()
#
# Description: we are closing down the adapter so close the logfile and
#   clean up.
# --------------------------------------------------------------------------

sub FinishSpecific
{
  local($logfile);

  package syslogd;

  foreach $logfile (keys %logfileTable) { close $logfile; }
}


###########################################
# sub specific routines
###########################################

# --------------------------------------------------------------------------
# Function: readLogFile()
#
# Description: do the work of reading a record from the log file
# --------------------------------------------------------------------------

sub readLogFile
{
  local($logfile) = @_;
  local($err, $n, $size, $inode, $curpos);
  local(@stat);

  if ($logfileTable{$logfile}{size} == -1)	# logfile doesn't exist?
  {
    if (-f $logfileTable{$logfile}{name})
    {
      open ($logfile, $logfileTable{$logfile}{name}) || &myDie("Cannot open logfile: $logfileTable{$logfile}{name}");

      @stat = stat($logfileTable{$logfile}{name});
      $logfileTable{$logfile}{size}  = 0;
      $logfileTable{$logfile}{inode} = $stat[1];
      seek($logfile, 0, 0);
    }
    else { return(0); }
  }

  # get the logfile info

  if (! open (TMPFH, $logfileTable{$logfile}{name}))
  {
    &Trace($Level{MINOR}, "Cannot open logfile: $logfileTable{$logfile}{name}");
    return(0);
  }

  @stat  = stat(TMPFH);
  $size  = $stat[7];
  $inode = $stat[1];
  close TMPFH;
	
  if ($inode != $logfileTable{$logfile}{inode})
  {
    &Trace($Level{MINOR}, "$logfile Inode changed\n");

    close $logfile;

    open ($logfile, $logfileTable{$logfile}{name}) || &myDie("Cannot open logfile: $logfileTable{$logfile}{name}");
    $logfileTable{$logfile}{size}  = 0;
    $logfileTable{$logfile}{inode} = $inode;
  }
  
  &Trace($Level{VERBOSE},"$logfile size previous:$logfileTable{$logfile}{size} actual:$size\n");

  $curpos = $size - $logfileTable{$logfile}{size};
  $curpos = $logfileTable{$logfile}{size};

  if ($size < $logfileTable{$logfile}{size}) { $curpos = -1; }

  if ($curpos < 0)	# logfile is smaller! Move to the start + start again
  {
    $logfileTable{$logfile}{size} = 0;
    seek($logfile, 0, 0);
  }

  # set the input record separator

  $/ = $formatTable{rs};

  # read

  seek($logfile, $curpos, 0);

  $_ = <$logfile>; 

  ####################################################
  #
  # check that we didn't read in the middle of a event

# $pat='^.*' . $/ . '$';

  # set the multi line match: 0 is no (the default) 1 is yes
 
  $* = $formatTable{multil};

# if ( ! /$pat/ )
# {
    # we read in the middle of an event, reset the multi line match,
    # seek back the previous position and return0.

#   $*=0;
#   seek($logfile, $logfileTable{$logfile}{size}, 0);
#   &Trace($Level{VERBOSE}, "read partial record\n");
#   return(0);
# }

  # reset the multi line match to NO
 
  $*=0;

  ####################################################
  # not sure I have to do that chop!?
  chop;

  # get the new position

  $curpos = tell($logfile);
  seek($logfile, $curpos, 0);

  &Trace($Level{VERBOSE},"$logfile current position: $curpos\n");

  # save the current position

  $logfileTable{$logfile}{size} = $curpos;

  &setEofOffset($CfgTable{EOFfile}, $curpos);

  # do not take care of empty line
  return(0) if ( /^[ \t]*$/ ) ;

  # &Trace($Level{VERBOSE}, "read the following:$_\n");

  return(1);
}

# --------------------------------------------------------------------------
# Function: createSpecificEvent()
#
# Description: Tokenize the event - take the event read and tokenize it.
# --------------------------------------------------------------------------

sub createSpecificEvent
{
  local($logfile) = @_;

  %specificEvent=();		# clear

  # tokenize the default tag

  $specificEvent{LOGFILE}  = $logfile;
  $specificEvent{LOGFILE}  = "FirewallLogfile";	# set a move useful name
  $specificEvent{complete} = $_;

  # call the adequat tokenize function

  if ($formatTable{regexpr}) { &tokenizeWithRegexpr($logfile, $_); }
  else                       { &tokenizeWithFS($logfile, $_); }
}

# --------------------------------------------------------------------------
# Function: tokenizeWithFS()
#
# Description: Tokenize based on a common field seperator
# --------------------------------------------------------------------------

sub tokenizeWithFS
{
  local($logfile, $buf) = @_;
  local($n);
  local(@fields);

  &Trace($Level{VERBOSE}, "tokenizeWithFS:$buf\n");

  # set the multi line match 0 is no (the default) 1 is yes
  # I am not sure it is usefull here!?
 
  $* = $formatTable{multil};

  # create a hash table using the specified separator
 
  ($err, @fields) = myquotewords($formatTable{fs}, 0, $buf);

  # reset the multi line match to NO
 
  $* = 0;

  return(0) if $err;

  # create the specific event

  for ($n = 0; $n < @fields; $n++)
  {
    $specificEvent{$formatTable{prefix} . $n} = $fields[$n];
  }
}

# --------------------------------------------------------------------------
# Function: tokenizeWithRegexpr()
#
# Description: Tokenize based on the regular expression defined above
# --------------------------------------------------------------------------

sub tokenizeWithRegexpr
{
  local($logfile, $buf) = @_;
  local($n);

# &Trace($Level{VERBOSE}, "tokenizeWithRegexpr:$buf\n");

  # set the multi line match 0 is no (the default) 1 is yes
 
  $* = $formatTable{multil};

  # eval the regular expression

  $buf =~ tr/\r//d;		# drop CR
  $buf =~ tr/"/'/d;		# translate double quotes to singles
  $buf =~ s/'/''/g;		# map single quotes to double single quotes
  $buf =~ s/[ \t\n]*$//g;	# purge trailing white space

  $buf=~/$formatTable{regexpr}/sm;

  &Trace($Level{VERBOSE}, "-->$formatTable{regexpr}<--\n");

  # create the specific event

  for ($n=1; $n <= $formatTable{nvar}; $n++)
  {
print "ok: -->$$n<--\n";
    &Trace($Level{VERBOSE},"$n -->$$n<--\n");
    $specificEvent{$formatTable{varname}[$n-1]} = $$n;
  }

  # the 'message' variable may contain newline characters
  # so replace the newline with the plus sign '+'

  $specificEvent{$formatTable{varname}[5]} =~ tr/\n/+/;

  # reset the multi line match to NO
 
  $*=0;
}

# --------------------------------------------------------------------------
# Function: setEofOffset()
#
# Description: record in a file the last record read
# --------------------------------------------------------------------------

sub setEofOffset
{
  local($filename, $offset) = @_;

  open(EOFFILE, "> $filename");
  print EOFFILE "$offset\n";
  close(EOFFILE);
}

# --------------------------------------------------------------------------
# Function: getEofOffset()
#
# Description: read the EOF file
# --------------------------------------------------------------------------

sub getEofOffset
{
  local($filename, *offset) = @_;

  open(EOFFILE, "< $filename");
  $offset = <EOFFILE>;
  chop($offset);

  close(EOFFILE);

  if ($offset eq "") { $offset = 0; }
}

#

;1
